'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});
exports.wsSend = wsSend;
exports.wsIsClosed = wsIsClosed;
exports.startWebsocket = startWebsocket;
exports.default = SmartWebsocket;
/**
 * Send message on ws if available.
 * feed the offlineBuffer if it is not
 */
function wsSend(ws, message, callback, offlinebuffer) {
	if (ws.readyState === WebSocket.OPEN) {
		ws.send(message);
		if (typeof callback === 'function') {
			callback();
		}
	} else {
		offlinebuffer.push({ message: message, callback: callback });
	}
}

/**
 * if is closed or not defined start a new ws
 */
function wsIsClosed(ws) {
	return !ws || ws.readyState === WebSocket.CLOSED;
}

/**
 * initialise a websocket
 * on open connection, try to send messages stored in offline buffer
 */
function startWebsocket(url, offlinebuffer, options) {
	var onMessage = options.onMessage,
	    onOpen = options.onOpen,
	    onClose = options.onClose,
	    onError = options.onError;

	var ws = new WebSocket(url);
	var pingInterval = void 0;
	ws.onopen = function onopen(event) {
		if (typeof onOpen === 'function') {
			onOpen(event);
		}
		// copy data in local array since offline buffer may be used in this process
		// in case we loose again ou connection
		var localBuffer = offlinebuffer.slice(0);
		offlinebuffer.reset();
		if (localBuffer.length > 0) {
			localBuffer.forEach(function (msg) {
				return wsSend(ws, msg.message, msg.callback, offlinebuffer);
			});
		}
		ws.ping();
		pingInterval = setInterval(ws.ping, 50000);
	};
	ws.onmessage = function onmessage(messageEvent) {
		if (typeof onMessage === 'function') {
			onMessage(messageEvent);
		}
	};
	ws.onclose = function onclose(closeEvent) {
		if (typeof onClose === 'function') {
			onClose(closeEvent);
		}
		if (pingInterval) {
			clearInterval(pingInterval);
		}
	};
	ws.onError = function onerror(event) {
		if (typeof onError === 'function') {
			onError(event);
		}
	};
	ws.ping = function ping() {
		ws.send('{"type":"PING"}');
	};
	return ws;
}

/**
 * create a new smart websocket featuring
 * automatic reconnection
 * message pooling in case of connection loss
 * @param ulr
 * @param options
 */
function SmartWebsocket(url) {
	var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

	var ws = void 0;
	var offlinebuffer = [];
	offlinebuffer.reset = function () {
		offlinebuffer.length = 0;
	};

	var send = function send(message, callback) {
		return wsSend(ws, message, callback, offlinebuffer);
	};
	var start = function start() {
		ws = startWebsocket(url, offlinebuffer, options);
	};

	start();
	var restartIfClosed = function restartIfClosed() {
		if (wsIsClosed(ws)) {
			start();
		}
	};

	var stop = setInterval(restartIfClosed, options.checkInterval || 5000);

	return {
		send: send,
		close: function close() {
			clearInterval(stop);
			return ws.close();
		},
		getBufferedAmount: function getBufferedAmount() {
			return ws.bufferedAmount;
		},
		getReadyState: function getReadyState() {
			return ws.readyState;
		},
		getUrl: function getUrl() {
			return ws.url;
		}
	};
}